import React from 'react';
import PropTypes from 'prop-types';
import DataGrid from './dataGrid/DataGrid';

// Styles
import getStyles from './ListView.styles';

/**
 * A control that displays array items in a tabular form
 * @see https://www.ag-grid.com/react-more-details/#ag-grid-react-overview
 */
class ListView extends React.Component {
    render() {
        const {
            rows,
            cols,
            multiSelectMode,
            contextStyles,
            onRowDblClick,
            testID,
        } = this.props;

        const styles = getStyles(contextStyles);

        return (
            <DataGrid
                testID={ testID }
                rows={ rows }
                cols={ cols }
                multiSelectMode={ multiSelectMode }
                contextStyles={ styles }
                onApplyFilters={ this.handleApplyFilters }
                onSortRequested={ this.handleSortRequested }
                onRowDblClick={ onRowDblClick } />
        );
    }

    /**
     * Handles sorting
     * @param {String} order direction of sorting, asc, or desc or ""
     */
    handleSortRequested = (order) => {
        const { onSortRequested } = this.props;
        return new Promise((resolve, reject) => {
            if (onSortRequested) {
                onSortRequested(order)
                    .then(() => resolve(order))
                    .catch((err) => reject(err));
            }
        });
    };

    /**
     * Handles filter application
     * @param {object} filterData column filter text and filter options
     */
    handleApplyFilters = (filterData) => {
        const { onApplyFilters } = this.props;
        return new Promise((resolve, reject) => {
            if (onApplyFilters) {
                onApplyFilters(filterData)
                    .then(() => resolve())
                    .catch((err) => reject(err));
            }
            else {
                reject('No method passed to handle filtering.');
            }
        });
    };
}

ListView.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the label and input toggle */
        container: PropTypes.object,

        /** Styles for a grid component */
        grid: PropTypes.object,

        /** Style object for header cell */
        headerCell: PropTypes.object,

        /** Style object for header text */
        headerText: PropTypes.object,

        /** Style object for grid row */
        row: PropTypes.object,

        /** Style object for selected grid row */
        selectedRow: PropTypes.object,

        /** Style object for oddRows*/
        oddRows: PropTypes.object,

        /** Style object for evenRows*/
        evenRows: PropTypes.object,

        /** Style object for grid filter dialog */
        filter: PropTypes.object,

        /** Style object for grid filter dialog container */
        filterContainer: PropTypes.object,

        /** Column level styles */
        column: PropTypes.object,
    }),

    /**
     * Action to run on row selection
     * @param {Event} event row details
     */
    onRowDblClick: PropTypes.func,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Array of records */
    rows: PropTypes.array.isRequired,

    /** Array containing column defintion object containing field(unique name for col), headerName(column header name), type(data type) and pinned(bool)*/
    cols: PropTypes.arrayOf(PropTypes.shape({
        /** field(unique name for col) */
        field: PropTypes.string,

        /** headerName(column header name) */
        headerName: PropTypes.string,

        /** Width of column */
        width: PropTypes.number,

        /** If column is pinned */
        pinned: PropTypes.bool,

        /** If column is sortable */
        sortable: PropTypes.bool,

        /**
         * Cell renderer
         * @param {Object} params column details for the cell
         */
        cellRenderer: PropTypes.func,
    })).isRequired,

    /** If multiple row selection is enabled */
    multiSelectMode: PropTypes.bool,

    /**
     *  Handle Sorting request
     *  @param {String} sortOrder direction of sorting, asc, or desc or ""
     */
    onSortRequested: PropTypes.func,

    /**
     *  Handle filter application on columns request
     *  @param {object} filterData columnwise filter text and filter options
     */
    onApplyFilters: PropTypes.func,
};

ListView.defaultProps = {
    contextStyles: {},
};

export default ListView;
